@@ -9,40 +9,37 @@ from urllib.parse import quote_plus, urlunparse
 
 from wykop.api.exceptions import WykopAPIError
 from wykop.api.api_const import CLIENT_NAME, DOMAIN, PROTOCOL
+from wykop.api.models.wykop_connect import WykopConnectLoginInfo
 from wykop.core.credentials import Credentials
 from wykop.core.parsers import default_parser
 from wykop.core.requesters import default_requester
-from wykop.utils import force_bytes, force_text, get_version, dictmap
+from wykop.utils import force_bytes, force_text, get_version, dictmap, sort_and_remove_none_values, \
+    validate_named_parameters, validate_api_parameters
 
 log = logging.getLogger(__name__)
 
 
 class Requestor:
 
-    def __init__(self, credentials: Credentials,
-                 output='', response_format='json'):
+    def __init__(self, credentials: Credentials, parser, output='', response_format='json'):
         self.credentials = credentials
         self.output = output
         self.format = response_format
         self.userkey = ''
         self.requester = default_requester
-        self.parser = default_parser
+        self.parser = parser
 
     def request(self, rtype, rmethod=None,
                 named_params=None, api_params=None, post_params=None, file_params=None):
         log.debug('Making request')
 
-        named_params = named_params or {}
-        api_params = api_params or []
-        post_params = OrderedDict({k: v for k, v in sorted(
-            post_params.items()) if v} if post_params else {})
+        post_params = sort_and_remove_none_values(post_params)
 
         file_params = file_params or {}
 
         rtype = force_text(rtype)
         rmethod = rmethod and force_text(rmethod)
         post_params = dictmap(force_bytes, post_params)
-        named_params = dictmap(force_text, named_params)
 
         url = self.construct_url(
             rtype=rtype, rmethod=rmethod, api_params=api_params, named_params=named_params)
@@ -54,7 +51,7 @@ class Requestor:
         if self.parser is None:
             return response
 
-        return self.parser.parse(response)
+        return self.parser.parse_response(response)
 
     def authenticate(self, account_key=None, login=None, password=None):
         self.credentials.account_key = account_key or self.credentials.account_key
@@ -89,6 +86,10 @@ class Requestor:
         post_params = {'code': tfa_token}
         return self.request('login', '2fa', post_params=post_params)
 
+    def wykop_connect_url(self, redirect_url: str = None):
+        named_params = self.connect_named_params(redirect_url)
+        return self.construct_url(rtype='Login', rmethod='Connect', named_params=named_params)
+
     def default_named_params(self):
         """
         Gets default named parameters.
@@ -158,10 +159,13 @@ class Requestor:
 
         return named_params
 
-    def construct_url(self, rtype, api_params, named_params, rmethod=None):
+    def construct_url(self, rtype, rmethod=None, api_params=None, named_params=None):
         """
         Constructs request url.
         """
+        named_params = dictmap(force_text, validate_named_parameters(named_params))
+        api_params = validate_api_parameters(api_params)
+
         path = self.path(rtype, api_params=api_params,
                          rmethod=rmethod, named_params=named_params)
 
@@ -198,3 +202,11 @@ class Requestor:
             for key, value in params.items()
             if value
         }
+
+    def valid_sign(self, connect_response: WykopConnectLoginInfo) -> bool:
+        secret_bytes = force_bytes(self.credentials.secretkey)
+        app_key_bytes = force_bytes(connect_response.app_key)
+        login_bytes = force_bytes(connect_response.login)
+        token_bytes = force_bytes(connect_response.token)
+        to_hash = secret_bytes + app_key_bytes + login_bytes + token_bytes
+        return hashlib.md5(to_hash).hexdigest() == connect_response.sign
